@@ -136,6 +136,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_in
         repr_data->struct_offsets      = (MVMint32 *)   MVM_malloc(info_alloc * sizeof(MVMint32));
         repr_data->flattened_stables   = (MVMSTable **) MVM_calloc(info_alloc, sizeof(MVMObject *));
         repr_data->member_types        = (MVMObject **) MVM_calloc(info_alloc, sizeof(MVMObject *));
+        repr_data->struct_align        = 0;
 
         /* Go over the attributes and arrange their allocation. */
         for (i = 0; i < num_attrs; i++) {
@@ -682,6 +683,7 @@ static void serialize_repr_data(MVMThreadContext *tc, MVMSTable *st, MVMSerializ
     MVMint32 i, num_classes, num_slots;
 
     MVM_serialization_write_varint(tc, writer, repr_data->struct_size);
+    MVM_serialization_write_varint(tc, writer, repr_data->struct_align);
     MVM_serialization_write_varint(tc, writer, repr_data->num_attributes);
     MVM_serialization_write_varint(tc, writer, repr_data->num_child_objs);
     for(i = 0; i < repr_data->num_attributes; i++){
@@ -721,6 +723,9 @@ static void deserialize_repr_data(MVMThreadContext *tc, MVMSTable *st, MVMSerial
     MVMint32 i, num_classes, num_slots;
 
     repr_data->struct_size = MVM_serialization_read_varint(tc, reader);
+    if (reader->root.version >= 17) {
+        repr_data->struct_align = MVM_serialization_read_varint(tc, reader);
+    }
     repr_data->num_attributes = MVM_serialization_read_varint(tc, reader);
     repr_data->num_child_objs = MVM_serialization_read_varint(tc, reader);
 
