@@ -288,15 +288,14 @@ char * MVM_exception_backtrace_line(MVMThreadContext *tc, MVMFrame *cur_frame, M
     char *o = MVM_malloc(1024);
     MVMuint8 *cur_op = not_top ? cur_frame->return_address : cur_frame->throw_address;
     MVMuint32 offset = cur_op - cur_frame->effective_bytecode;
-    MVMuint32 instr = MVM_bytecode_offset_to_instr_idx(tc, cur_frame->static_info, offset);
     MVMBytecodeAnnotation *annot = MVM_bytecode_resolve_annotation(tc, &cur_frame->static_info->body,
                                         offset > 0 ? offset - 1 : 0);
 
     MVMuint32 line_number = annot ? annot->line_number : 1;
     MVMuint16 string_heap_index = annot ? annot->filename_string_heap_index : 0;
     char *tmp1 = annot && string_heap_index < cur_frame->static_info->body.cu->body.num_strings
-        ? MVM_string_utf8_encode_C_string(tc,
-            cur_frame->static_info->body.cu->body.strings[string_heap_index])
+        ? MVM_string_utf8_encode_C_string(tc, MVM_cu_string(tc,
+                cur_frame->static_info->body.cu, string_heap_index))
         : NULL;
 
     char *filename_c = filename
@@ -306,18 +305,12 @@ char * MVM_exception_backtrace_line(MVMThreadContext *tc, MVMFrame *cur_frame, M
 	? MVM_string_utf8_encode_C_string(tc, name)
 	: "<anonymous frame>";
 
-    /* We may be mid-instruction if exception was thrown at an unfortunate
-     * point; try to cope with that. */
-    if (instr == MVM_BC_ILLEGAL_OFFSET && offset >= 2)
-        instr = MVM_bytecode_offset_to_instr_idx(tc, cur_frame->static_info, offset - 2);
-
-    snprintf(o, 1024, " %s %s:%u  (%s:%s:%u)",
+    snprintf(o, 1024, " %s %s:%u  (%s:%s)",
         not_top ? "from" : "  at",
         tmp1 ? tmp1 : "<unknown>",
         line_number,
         filename_c,
-        name_c,
-        instr
+        name_c
     );
     if (filename)
 	MVM_free(filename_c);
@@ -375,10 +368,10 @@ MVMObject * MVM_exception_backtrace(MVMThreadContext *tc, MVMObject *ex_obj) {
         /* file */
         if (fshi >= 0 && fshi < cur_frame->static_info->body.cu->body.num_strings)
             value = MVM_repr_box_str(tc, MVM_hll_current(tc)->str_box_type,
-                        cur_frame->static_info->body.cu->body.strings[fshi]);
+                MVM_cu_string(tc, cur_frame->static_info->body.cu, fshi));
         else
             value = MVM_repr_box_str(tc, MVM_hll_current(tc)->str_box_type,
-                        cur_frame->static_info->body.cu->body.filename);
+                cur_frame->static_info->body.cu->body.filename);
         MVM_repr_bind_key_o(tc, annotations, k_file, value);
 
         /* line */
