@@ -64,9 +64,13 @@ void MVM_gc_collect(MVMThreadContext *tc, MVMuint8 what_to_do, MVMuint8 gen) {
         process_worklist(tc, worklist, &wtp, gen);
     }
     else {
-        /* Main collection run. Swap fromspace and tospace. */
-        void * fromspace = tc->nursery_tospace;
-        void * tospace   = tc->nursery_fromspace;
+        /* Main collection run. Swap fromspace and tospace, allocating the
+         * new tospace if that didn't yet happen (we don't allocate it at
+         * startup, to cut memory use for threads that quit before a GC). */
+        void *fromspace = tc->nursery_tospace;
+        void *tospace   = tc->nursery_fromspace;
+        if (!tospace)
+            tospace = MVM_calloc(1, MVM_NURSERY_SIZE);
         tc->nursery_fromspace = fromspace;
         tc->nursery_tospace   = tospace;
 
@@ -119,8 +123,8 @@ void MVM_gc_collect(MVMThreadContext *tc, MVMuint8 what_to_do, MVMuint8 gen) {
         if (tc->cur_frame) {
             MVM_gc_worklist_add_frame(tc, worklist, tc->cur_frame);
             GCDEBUG_LOG(tc, MVM_GC_DEBUG_COLLECT, "Thread %d run %d : processing %d items from cur_frame \n", worklist->items);
-			process_worklist(tc, worklist, &wtp, gen);
-		}
+            process_worklist(tc, worklist, &wtp, gen);
+        }
 
         /* Process anything in the in-tray. */
         add_in_tray_to_worklist(tc, worklist);
@@ -413,7 +417,7 @@ static void push_work_to_thread_in_tray(MVMThreadContext *tc, MVMuint32 target,
             }
         } while ((t = t->body.next));
         if (!target_tc)
-            MVM_panic(MVM_exitcode_gcnursery, "Internal error: invalid thread ID in GC work pass");
+            MVM_panic(MVM_exitcode_gcnursery, "Internal error: invalid thread ID %d in GC work pass", target);
     }
 
     /* Pass the work, chaining any other in-tray entries for the thread
@@ -599,7 +603,7 @@ void MVM_gc_collect_free_stables(MVMThreadContext *tc) {
 
 /* Goes through the unmarked objects in the second generation heap and builds
  * free lists out of them. Also does any required finalization. */
-void MVM_gc_collect_free_gen2_unmarked(MVMThreadContext *tc) {
+void MVM_gc_collect_free_gen2_unmarked(MVMThreadContext *tc, MVMint32 global_destruction) {
     /* Visit each of the size class bins. */
     MVMGen2Allocator *gen2 = tc->gen2;
     MVMuint32 bin, obj_size, page, i;
@@ -682,7 +686,7 @@ void MVM_gc_collect_free_gen2_unmarked(MVMThreadContext *tc) {
                                 col->flags &= ~MVM_CF_SERIALZATION_INDEX_ALLOCATED;
                             }
 #endif
-                            if (MVM_load(&tc->gc_status) == MVMGCStatus_NONE) {
+                            if (global_destruction) {
                                 /* We're in global destruction, so enqueue to the end
                                  * like we do in the nursery */
                                 MVM_gc_collect_enqueue_stable_for_deletion(tc, (MVMSTable *)col);
