@@ -1,6 +1,10 @@
 #include "moar.h"
 #include <math.h>
 
+#ifndef MAX
+    #define MAX(x,y) ((x)>(y)?(x):(y))
+#endif
+
 /* Taken from mp_set_long, but portably accepts a 64-bit number. */
 int MVM_bigint_mp_set_uint64(mp_int * a, MVMuint64 b) {
   int     x, res;
@@ -114,7 +118,7 @@ static MVMP6bigintBody * get_bigint_body(MVMThreadContext *tc, MVMObject *obj) {
 }
 
 /* Checks if a bigint can be stored small. */
-static int can_be_smallint(mp_int *i) {
+static int can_be_smallint(const mp_int *i) {
     if (USED(i) != 1)
         return 0;
     return MVM_IS_32BIT_INT(DIGIT(i, 0));
@@ -122,7 +126,7 @@ static int can_be_smallint(mp_int *i) {
 
 /* Forces a bigint, even if we only have a smallint. Takes a parameter that
  * indicates where to allocate a temporary mp_int if needed. */
-static mp_int * force_bigint(MVMP6bigintBody *body, mp_int **tmp) {
+static mp_int * force_bigint(const MVMP6bigintBody *body, mp_int **tmp) {
     if (MVM_BIGINT_IS_BIG(body)) {
         return body->u.bigint;
     }
@@ -145,7 +149,7 @@ static mp_int * force_bigint(MVMP6bigintBody *body, mp_int **tmp) {
 }
 
 /* Clears an array that may contain tempory big ints. */
-static void clear_temp_bigints(mp_int **ints, MVMint32 n) {
+static void clear_temp_bigints(mp_int *const *ints, MVMint32 n) {
     MVMint32 i;
     for (i = 0; i < n; i++)
         if (ints[i]) {
@@ -198,7 +202,7 @@ static void store_bigint_result(MVMP6bigintBody *body, mp_int *i) {
  * appends sign-bit extension DIGITs to it to give us a 2s compliment
  * representation in memory.  Do not call it on positive bigints.
  */
-static void grow_and_negate(mp_int *a, int size, mp_int *b) {
+static void grow_and_negate(const mp_int *a, int size, mp_int *b) {
     int i;
     /* Always add an extra DIGIT so we can tell positive values
      * with a one in the highest bit apart from negative values.
@@ -372,10 +376,17 @@ MVMObject * MVM_bigint_##opname(MVMThreadContext *tc, MVMObject *result_type, MV
 }
 
 #define MVM_BIGINT_BINARY_OP_2(opname, SMALLINT_OP) \
-void MVM_bigint_##opname(MVMThreadContext *tc, MVMObject *result, MVMObject *a, MVMObject *b) { \
+MVMObject * MVM_bigint_##opname(MVMThreadContext *tc, MVMObject *result_type, MVMObject *a, MVMObject *b) { \
     MVMP6bigintBody *ba = get_bigint_body(tc, a); \
     MVMP6bigintBody *bb = get_bigint_body(tc, b); \
-    MVMP6bigintBody *bc = get_bigint_body(tc, result); \
+    MVMP6bigintBody *bc; \
+    MVMObject *result; \
+    MVMROOT(tc, a, { \
+    MVMROOT(tc, b, { \
+        result = MVM_repr_alloc_init(tc, result_type);\
+    }); \
+    }); \
+    bc = get_bigint_body(tc, result); \
     if (MVM_BIGINT_IS_BIG(ba) || MVM_BIGINT_IS_BIG(bb)) { \
         mp_int *tmp[2] = { NULL, NULL }; \
         mp_int *ia = force_bigint(ba, tmp); \
@@ -393,6 +404,7 @@ void MVM_bigint_##opname(MVMThreadContext *tc, MVMObject *result, MVMObject *a,
         SMALLINT_OP; \
         store_int64_result(bc, sc); \
     } \
+    return result; \
 }
 
 MVM_BIGINT_UNARY_OP(abs, { sb = labs(sa); })
@@ -582,20 +594,20 @@ MVMObject *MVM_bigint_div(MVMThreadContext *tc, MVMObject *result_type, MVMObjec
     } else {
         MVMint32 num   = ba->u.smallint.value;
         MVMint32 denom = bb->u.smallint.value;
-        MVMint32 result;
+        MVMint32 value;
         if ((cmp_a == MP_LT) ^ (cmp_b == MP_LT)) {
             if (denom == 0) {
                 MVM_exception_throw_adhoc(tc, "Division by zero");
             }
             if ((num % denom) != 0) {
-                result = num / denom - 1;
+                value = num / denom - 1;
             } else {
-                result = num / denom;
+                value = num / denom;
             }
         } else {
-            result = num / denom;
+            value = num / denom;
         }
-        store_int64_result(bc, result);
+        store_int64_result(bc, value);
     }
 
     return result;
@@ -650,9 +662,17 @@ MVMObject * MVM_bigint_pow(MVMThreadContext *tc, MVMObject *a, MVMObject *b,
     return r;
 }
 
-void MVM_bigint_shl(MVMThreadContext *tc, MVMObject *result, MVMObject *a, MVMint64 n) {
+MVMObject *MVM_bigint_shl(MVMThreadContext *tc, MVMObject *result_type, MVMObject *a, MVMint64 n) {
     MVMP6bigintBody *ba = get_bigint_body(tc, a);
-    MVMP6bigintBody *bb = get_bigint_body(tc, result);
+    MVMP6bigintBody *bb;
+    MVMObject       *result;
+
+    MVMROOT(tc, a, {
+        result = MVM_repr_alloc_init(tc, result_type);
+    });
+
+    bb = get_bigint_body(tc, result);
+
     if (MVM_BIGINT_IS_BIG(ba) || n >= 31) {
         mp_int *tmp[1] = { NULL };
         mp_int *ia = force_bigint(ba, tmp);
@@ -662,18 +682,28 @@ void MVM_bigint_shl(MVMThreadContext *tc, MVMObject *result, MVMObject *a, MVMin
         store_bigint_result(bb, ib);
         clear_temp_bigints(tmp, 1);
     } else {
-        MVMint64 result;
+        MVMint64 value;
         if (n < 0)
-            result = ((MVMint64)ba->u.smallint.value) >> -n;
+            value = ((MVMint64)ba->u.smallint.value) >> -n;
         else
-            result = ((MVMint64)ba->u.smallint.value) << n;
-        store_int64_result(bb, result);
+            value = ((MVMint64)ba->u.smallint.value) << n;
+        store_int64_result(bb, value);
     }
+
+    return result;
 }
 
-void MVM_bigint_shr(MVMThreadContext *tc, MVMObject *result, MVMObject *a, MVMint64 n) {
+MVMObject *MVM_bigint_shr(MVMThreadContext *tc, MVMObject *result_type, MVMObject *a, MVMint64 n) {
     MVMP6bigintBody *ba = get_bigint_body(tc, a);
-    MVMP6bigintBody *bb = get_bigint_body(tc, result);
+    MVMP6bigintBody *bb;
+    MVMObject       *result;
+
+    MVMROOT(tc, a, {
+        result = MVM_repr_alloc_init(tc, result_type);
+    });
+
+    bb = get_bigint_body(tc, result);
+
     if (MVM_BIGINT_IS_BIG(ba) || n < 0) {
         mp_int *tmp[1] = { NULL };
         mp_int *ia = force_bigint(ba, tmp);
@@ -686,14 +716,24 @@ void MVM_bigint_shr(MVMThreadContext *tc, MVMObject *result, MVMObject *a, MVMin
         store_int64_result(bb, 0);
     } else {
         MVMint32 value = ba->u.smallint.value;
-        MVMint32 result = value >> n;
-        store_int64_result(bb, result);
+        value = value >> n;
+        store_int64_result(bb, value);
     }
+
+    return result;
 }
 
-void MVM_bigint_not(MVMThreadContext *tc, MVMObject *result, MVMObject *a) {
+MVMObject *MVM_bigint_not(MVMThreadContext *tc, MVMObject *result_type, MVMObject *a) {
     MVMP6bigintBody *ba = get_bigint_body(tc, a);
-    MVMP6bigintBody *bb = get_bigint_body(tc, result);
+    MVMP6bigintBody *bb;
+    MVMObject       *result;
+
+    MVMROOT(tc, a, {
+        result = MVM_repr_alloc_init(tc, result_type);
+    });
+
+    bb = get_bigint_body(tc, result);
+
     if (MVM_BIGINT_IS_BIG(ba)) {
         mp_int *ia = ba->u.bigint;
         mp_int *ib = MVM_malloc(sizeof(mp_int));
@@ -704,9 +744,11 @@ void MVM_bigint_not(MVMThreadContext *tc, MVMObject *result, MVMObject *a) {
         store_bigint_result(bb, ib);
     } else {
         MVMint32 value = ba->u.smallint.value;
-        MVMint32 result = ~value;
-        store_int64_result(bb, result);
+        value = ~value;
+        store_int64_result(bb, value);
     }
+
+    return result;
 }
 
 void MVM_bigint_expmod(MVMThreadContext *tc, MVMObject *result, MVMObject *a, MVMObject *b, MVMObject *c) {
@@ -728,7 +770,7 @@ void MVM_bigint_expmod(MVMThreadContext *tc, MVMObject *result, MVMObject *a, MV
     clear_temp_bigints(tmp, 3);
 }
 
-void MVM_bigint_from_str(MVMThreadContext *tc, MVMObject *a, char *buf) {
+void MVM_bigint_from_str(MVMThreadContext *tc, MVMObject *a, const char *buf) {
     MVMP6bigintBody *body = get_bigint_body(tc, a);
     mp_int *i = MVM_malloc(sizeof(mp_int));
     mp_init(i);
@@ -951,7 +993,7 @@ MVMObject * MVM_bigint_radix(MVMThreadContext *tc, MVMint64 radix, MVMString *st
         else if (ch >= 'A' && ch <= 'Z') ch = ch - 'A' + 10;
         else if (ch >= 0xFF21 && ch <= 0xFF3A) ch = ch - 0xFF21 + 10; /* uppercase fullwidth */
         else if (ch >= 0xFF41 && ch <= 0xFF5A) ch = ch - 0xFF41 + 10; /* lowercase fullwidth */
-        else if (MVM_unicode_codepoint_has_property_value(tc, ch, MVM_UNICODE_PROPERTY_GENERAL_CATEGORY,
+        else if (ch > 0 && MVM_unicode_codepoint_has_property_value(tc, ch, MVM_UNICODE_PROPERTY_GENERAL_CATEGORY,
                 MVM_unicode_cname_to_property_value_code(tc, MVM_UNICODE_PROPERTY_GENERAL_CATEGORY, STR_WITH_LEN("Nd")))) {
             /* as of Unicode 6.0.0, characters with the 'de' Numeric Type (and are
              * thus also of General Category Nd, since 4.0.0) are contiguous
