@@ -279,8 +279,6 @@ static void toplevel_initial_invoke(MVMThreadContext *tc, void *data) {
 
 /* Loads bytecode from the specified file name and runs it. */
 void MVM_vm_run_file(MVMInstance *instance, const char *filename) {
-    MVMStaticFrame *start_frame;
-
     /* Map the compilation unit into memory and dissect it. */
     MVMThreadContext *tc = instance->main_thread;
     MVMCompUnit      *cu = MVM_cu_map_from_file(tc, filename);
@@ -297,10 +295,8 @@ void MVM_vm_run_file(MVMInstance *instance, const char *filename) {
         }
     });
 
-    /* Run the frame marked main, or if there is none then fall back to the
-     * first frame. */
-    start_frame = cu->body.main_frame ? cu->body.main_frame : cu->body.frames[0];
-    MVM_interp_run(tc, toplevel_initial_invoke, start_frame);
+    /* Run the entry-point frame. */
+    MVM_interp_run(tc, toplevel_initial_invoke, cu->body.main_frame);
 }
 
 /* Loads bytecode from the specified file name and dumps it. */
@@ -309,8 +305,22 @@ void MVM_vm_dump_file(MVMInstance *instance, const char *filename) {
     MVMThreadContext *tc = instance->main_thread;
     MVMCompUnit      *cu = MVM_cu_map_from_file(tc, filename);
     char *dump = MVM_bytecode_dump(tc, cu);
+    size_t dumplen = strlen(dump);
+    int position = 0;
+
+    /* libuv already set up stdout to be nonblocking, but it can very well be
+     * we encounter EAGAIN (Resource temporarily unavailable), so we need to
+     * loop over our buffer, which can be quite big.
+     *
+     * The CORE.setting.moarvm has - as of writing this - about 32 megs of
+     * output from dumping.
+     */
+    while (position < dumplen) {
+        size_t written = write(1, dump + position, dumplen - position);
+        if (written > 0)
+            position += written;
+    }
 
-    printf("%s", dump);
     MVM_free(dump);
 }
 
