@@ -10,6 +10,9 @@
 /* Standard integer types. */
 #include <platform/inttypes.h>
 
+/* platform-specific setjmp override */
+#include <platform/setjmp.h>
+
 /* stuff for uthash */
 #define uthash_fatal(msg) MVM_exception_throw_adhoc(tc, "internal hash error: " msg)
 
@@ -60,6 +63,9 @@ typedef double   MVMnum64;
 #define ALIGNOF(t) ((char *)(&((struct { char c; t _h; } *)0)->_h) - (char *)0)
 #endif
 
+#define MVM_ALIGN_SECTION_MASK ((MVMuint32)ALIGNOF(MVMint64) - 1)
+#define MVM_ALIGN_SECTION(offset) (((offset) + (MVM_ALIGN_SECTION_MASK)) & ~(MVM_ALIGN_SECTION_MASK))
+
 #if defined MVM_BUILD_SHARED
 #  define MVM_PUBLIC  MVM_DLL_EXPORT
 #  define MVM_PRIVATE MVM_DLL_LOCAL
@@ -95,7 +101,6 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "core/alloc.h"
 #include "core/frame.h"
 #include "core/validation.h"
-#include "core/compunit.h"
 #include "core/bytecode.h"
 #include "core/bytecodedump.h"
 #include "core/ops.h"
@@ -120,6 +125,7 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "6model/sc.h"
 #include "6model/serialization.h"
 #include "6model/parametric.h"
+#include "core/compunit.h"
 #include "gc/gen2.h"
 #include "gc/allocation.h"
 #include "gc/worklist.h"
@@ -145,6 +151,7 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "strings/decode_stream.h"
 #include "strings/ascii.h"
 #include "strings/utf8.h"
+#include "strings/utf8_c8.h"
 #include "strings/utf16.h"
 #include "strings/nfg.h"
 #include "strings/iter.h"
@@ -166,6 +173,7 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "io/filewatchers.h"
 #include "io/signals.h"
 #include "io/asyncsocket.h"
+#include "io/asyncsocketudp.h"
 #include "math/bigintops.h"
 #include "mast/driver.h"
 #include "core/intcache.h"
