@@ -68,6 +68,16 @@ static void instrument_graph(MVMThreadContext *tc, MVMSpeshGraph *g) {
 
                 break;
             }
+            case MVM_OP_invoke_o:
+            case MVM_OP_param_rp_o:
+            case MVM_OP_param_op_o:
+            case MVM_OP_param_rn_o:
+            case MVM_OP_param_on_o:
+            case MVM_OP_param_sp:
+            case MVM_OP_param_sn:
+            case MVM_OP_newexception:
+            case MVM_OP_usecapture:
+            case MVM_OP_savecapture:
             case MVM_OP_takeclosure:
             case MVM_OP_getattr_o:
             case MVM_OP_getattrs_o:
@@ -168,15 +178,18 @@ static void instrument_graph(MVMThreadContext *tc, MVMSpeshGraph *g) {
 /* Adds instrumented versions of the unspecialized bytecode. */
 static void add_instrumentation(MVMThreadContext *tc, MVMStaticFrame *sf) {
     MVMSpeshCode  *sc;
+    MVMStaticFrameInstrumentation *ins;
     MVMSpeshGraph *sg = MVM_spesh_graph_create(tc, sf, 1);
     instrument_graph(tc, sg);
     sc = MVM_spesh_codegen(tc, sg);
-    sf->body.instrumented_bytecode        = sc->bytecode;
-    sf->body.instrumented_handlers        = sc->handlers;
-    sf->body.instrumented_bytecode_size   = sc->bytecode_size;
-    sf->body.uninstrumented_bytecode      = sf->body.bytecode;
-    sf->body.uninstrumented_handlers      = sf->body.handlers;
-    sf->body.uninstrumented_bytecode_size = sf->body.bytecode_size;
+    ins = MVM_calloc(1, sizeof(MVMStaticFrameInstrumentation));
+    ins->instrumented_bytecode        = sc->bytecode;
+    ins->instrumented_handlers        = sc->handlers;
+    ins->instrumented_bytecode_size   = sc->bytecode_size;
+    ins->uninstrumented_bytecode      = sf->body.bytecode;
+    ins->uninstrumented_handlers      = sf->body.handlers;
+    ins->uninstrumented_bytecode_size = sf->body.bytecode_size;
+    sf->body.instrumentation = ins;
     MVM_spesh_graph_destroy(tc, sg);
     MVM_free(sc);
 }
@@ -184,13 +197,13 @@ static void add_instrumentation(MVMThreadContext *tc, MVMStaticFrame *sf) {
 /* Instruments a static frame for profiling, or uses an existing
  * instrumentation if it exists. */
 void MVM_profile_instrument(MVMThreadContext *tc, MVMStaticFrame *sf) {
-    if (sf->body.bytecode != sf->body.instrumented_bytecode) {
+    if (!sf->body.instrumentation || sf->body.bytecode != sf->body.instrumentation->instrumented_bytecode) {
         /* Handle main, non-specialized, bytecode. */
-        if (!sf->body.instrumented_bytecode)
+        if (!sf->body.instrumentation)
             add_instrumentation(tc, sf);
-        sf->body.bytecode      = sf->body.instrumented_bytecode;
-        sf->body.handlers      = sf->body.instrumented_handlers;
-        sf->body.bytecode_size = sf->body.instrumented_bytecode_size;
+        sf->body.bytecode      = sf->body.instrumentation->instrumented_bytecode;
+        sf->body.handlers      = sf->body.instrumentation->instrumented_handlers;
+        sf->body.bytecode_size = sf->body.instrumentation->instrumented_bytecode_size;
 
         /* Throw away any specializations; we'll need to reproduce them as
          * instrumented versions. */
@@ -201,11 +214,11 @@ void MVM_profile_instrument(MVMThreadContext *tc, MVMStaticFrame *sf) {
 
 /* Ensures we're no longer in instrumented code. */
 void MVM_profile_ensure_uninstrumented(MVMThreadContext *tc, MVMStaticFrame *sf) {
-    if (sf->body.bytecode == sf->body.instrumented_bytecode) {
+    if (sf->body.instrumentation && sf->body.bytecode == sf->body.instrumentation->instrumented_bytecode) {
         /* Switch to uninstrumented code. */
-        sf->body.bytecode      = sf->body.uninstrumented_bytecode;
-        sf->body.handlers      = sf->body.uninstrumented_handlers;
-        sf->body.bytecode_size = sf->body.uninstrumented_bytecode_size;
+        sf->body.bytecode      = sf->body.instrumentation->uninstrumented_bytecode;
+        sf->body.handlers      = sf->body.instrumentation->uninstrumented_handlers;
+        sf->body.bytecode_size = sf->body.instrumentation->uninstrumented_bytecode_size;
 
         /* Throw away specializations, which may also be instrumented. */
         sf->body.num_spesh_candidates = 0;
