@@ -48,63 +48,89 @@ typedef struct {
     MVMString *deopt_one;
     MVMString *deopt_all;
     MVMString *spesh_time;
+    MVMString *native_lib;
 } ProfDumpStrs;
 
 /* Dumps a call graph node. */
 static MVMObject * dump_call_graph_node(MVMThreadContext *tc, ProfDumpStrs *pds,
-                                        MVMProfileCallNode *pcn) {
+                                        const MVMProfileCallNode *pcn) {
     MVMObject *node_hash  = new_hash(tc);
-    MVMObject *alloc_list = new_array(tc);
     MVMuint32  i;
 
-    /* Try to resolve the code filename and line number. */
-    MVMBytecodeAnnotation *annot = MVM_bytecode_resolve_annotation(tc,
-        &(pcn->sf->body), 0);
-    MVMint32 fshi = annot ? (MVMint32)annot->filename_string_heap_index : -1;
+    /* Let's see if we're dealing with a native call or a regular moar call */
+    if (pcn->sf) {
+        /* Try to resolve the code filename and line number. */
+        MVMBytecodeAnnotation *annot = MVM_bytecode_resolve_annotation(tc,
+            &(pcn->sf->body), 0);
+        MVMint32 fshi = annot ? (MVMint32)annot->filename_string_heap_index : -1;
 
-    /* Add name of code object. */
-    MVM_repr_bind_key_o(tc, node_hash,
-        pds->name, box_s(tc, pcn->sf->body.name));
+        /* Add name of code object. */
+        MVM_repr_bind_key_o(tc, node_hash, pds->name,
+            box_s(tc, pcn->sf->body.name));
 
-    /* Add line number and file name. */
-    if (fshi >= 0 && fshi < pcn->sf->body.cu->body.num_strings)
-        MVM_repr_bind_key_o(tc, node_hash, pds->file,
-            box_s(tc, pcn->sf->body.cu->body.strings[fshi]));
-    else if (pcn->sf->body.cu->body.filename)
-        MVM_repr_bind_key_o(tc, node_hash, pds->file,
-            box_s(tc, pcn->sf->body.cu->body.filename));
-    else
-        MVM_repr_bind_key_o(tc, node_hash, pds->file,
-            box_s(tc, tc->instance->str_consts.empty));
-    MVM_repr_bind_key_o(tc, node_hash, pds->line,
-        box_i(tc, annot ? (MVMint32)annot->line_number : -1));
-    MVM_free(annot);
+        /* Add line number and file name. */
+        if (fshi >= 0 && fshi < pcn->sf->body.cu->body.num_strings)
+            MVM_repr_bind_key_o(tc, node_hash, pds->file,
+                box_s(tc, MVM_cu_string(tc, pcn->sf->body.cu, fshi)));
+        else if (pcn->sf->body.cu->body.filename)
+            MVM_repr_bind_key_o(tc, node_hash, pds->file,
+                box_s(tc, pcn->sf->body.cu->body.filename));
+        else
+            MVM_repr_bind_key_o(tc, node_hash, pds->file,
+                box_s(tc, tc->instance->str_consts.empty));
+        MVM_repr_bind_key_o(tc, node_hash, pds->line,
+            box_i(tc, annot ? (MVMint32)annot->line_number : -1));
+        MVM_free(annot);
 
-    /* Use static frame memory address to get a unique ID. */
-    MVM_repr_bind_key_o(tc, node_hash, pds->id,
-        box_i(tc, (MVMint64)pcn->sf));
+        /* Use static frame memory address to get a unique ID. */
+        MVM_repr_bind_key_o(tc, node_hash, pds->id,
+            box_i(tc, (MVMint64)pcn->sf));
+    } else {
+        MVMString *function_name_string =
+            MVM_string_utf8_c8_decode(tc, tc->instance->VMString,
+                                      pcn->native_target_name, strlen(pcn->native_target_name));
+
+        MVM_repr_bind_key_o(tc, node_hash, pds->name,
+            box_s(tc, function_name_string));
+        MVM_repr_bind_key_o(tc, node_hash, pds->file,
+            box_s(tc, pds->native_lib));
+
+        MVM_repr_bind_key_o(tc, node_hash, pds->line,
+            box_i(tc, -2));
+
+        /* Use the address of the name string as unique ID. a hack, but oh well. */
+        MVM_repr_bind_key_o(tc, node_hash, pds->id,
+            box_i(tc, (MVMint64)pcn->native_target_name));
+    }
 
     /* Entry counts. */
-    MVM_repr_bind_key_o(tc, node_hash, pds->entries,
-        box_i(tc, pcn->total_entries));
-    MVM_repr_bind_key_o(tc, node_hash, pds->spesh_entries,
-        box_i(tc, pcn->specialized_entries));
-    MVM_repr_bind_key_o(tc, node_hash, pds->jit_entries,
-        box_i(tc, pcn->jit_entries));
-    MVM_repr_bind_key_o(tc, node_hash, pds->inlined_entries,
-        box_i(tc, pcn->inlined_entries));
+    if (pcn->total_entries)
+        MVM_repr_bind_key_o(tc, node_hash, pds->entries,
+            box_i(tc, pcn->total_entries));
+    if (pcn->specialized_entries)
+        MVM_repr_bind_key_o(tc, node_hash, pds->spesh_entries,
+            box_i(tc, pcn->specialized_entries));
+    if (pcn->jit_entries)
+        MVM_repr_bind_key_o(tc, node_hash, pds->jit_entries,
+            box_i(tc, pcn->jit_entries));
+    if (pcn->inlined_entries)
+        MVM_repr_bind_key_o(tc, node_hash, pds->inlined_entries,
+            box_i(tc, pcn->inlined_entries));
 
     /* Total (inclusive) time. */
     MVM_repr_bind_key_o(tc, node_hash, pds->inclusive_time,
         box_i(tc, pcn->total_time / 1000));
 
     /* OSR and deopt counts. */
-    MVM_repr_bind_key_o(tc, node_hash, pds->osr,
-        box_i(tc, pcn->osr_count));
-    MVM_repr_bind_key_o(tc, node_hash, pds->deopt_one,
-        box_i(tc, pcn->deopt_one_count));
-    MVM_repr_bind_key_o(tc, node_hash, pds->deopt_all,
-        box_i(tc, pcn->deopt_all_count));
+    if (pcn->osr_count)
+        MVM_repr_bind_key_o(tc, node_hash, pds->osr,
+            box_i(tc, pcn->osr_count));
+    if (pcn->deopt_one_count)
+        MVM_repr_bind_key_o(tc, node_hash, pds->deopt_one,
+            box_i(tc, pcn->deopt_one_count));
+    if (pcn->deopt_all_count)
+        MVM_repr_bind_key_o(tc, node_hash, pds->deopt_all,
+            box_i(tc, pcn->deopt_all_count));
 
     /* Visit successors in the call graph, dumping them and working out the
      * exclusive time. */
@@ -125,22 +151,30 @@ static MVMObject * dump_call_graph_node(MVMThreadContext *tc, ProfDumpStrs *pds,
             box_i(tc, pcn->total_time / 1000));
     }
 
-    /* Emit allocations. */
-    MVM_repr_bind_key_o(tc, node_hash, pds->allocations, alloc_list);
-    for (i = 0; i < pcn->num_alloc; i++) {
-        MVMObject *alloc_info = new_hash(tc);
-        MVMObject *type       = pcn->alloc[i].type;
-        MVM_repr_bind_key_o(tc, alloc_info, pds->id, box_i(tc, (MVMint64)type));
-        MVM_repr_bind_key_o(tc, alloc_info, pds->type, type);
-        MVM_repr_bind_key_o(tc, alloc_info, pds->spesh,
-            box_i(tc, pcn->alloc[i].allocations_spesh));
-        MVM_repr_bind_key_o(tc, alloc_info, pds->jit,
-            box_i(tc, pcn->alloc[i].allocations_jit));
-        MVM_repr_bind_key_o(tc, alloc_info, pds->count,
-            box_i(tc, pcn->alloc[i].allocations_interp
-                      + pcn->alloc[i].allocations_spesh
-                      + pcn->alloc[i].allocations_jit));
-        MVM_repr_push_o(tc, alloc_list, alloc_info);
+    if (pcn->num_alloc) {
+        /* Emit allocations. */
+        MVMObject *alloc_list = new_array(tc);
+        MVM_repr_bind_key_o(tc, node_hash, pds->allocations, alloc_list);
+        for (i = 0; i < pcn->num_alloc; i++) {
+            MVMObject *alloc_info = new_hash(tc);
+            MVMProfileAllocationCount *alloc = &pcn->alloc[i];
+
+            MVMObject *type       = pcn->alloc[i].type;
+
+            MVM_repr_bind_key_o(tc, alloc_info, pds->id, box_i(tc, (MVMint64)type));
+            MVM_repr_bind_key_o(tc, alloc_info, pds->type, type);
+            if (alloc->allocations_spesh)
+                MVM_repr_bind_key_o(tc, alloc_info, pds->spesh,
+                    box_i(tc, alloc->allocations_spesh));
+            if (alloc->allocations_jit)
+                MVM_repr_bind_key_o(tc, alloc_info, pds->jit,
+                    box_i(tc, alloc->allocations_jit));
+            MVM_repr_bind_key_o(tc, alloc_info, pds->count,
+                box_i(tc, alloc->allocations_interp
+                          + alloc->allocations_spesh
+                          + alloc->allocations_jit));
+            MVM_repr_push_o(tc, alloc_list, alloc_info);
+        }
     }
 
     return node_hash;
@@ -148,7 +182,7 @@ static MVMObject * dump_call_graph_node(MVMThreadContext *tc, ProfDumpStrs *pds,
 
 /* Dumps data from a single thread. */
 static MVMObject * dump_thread_data(MVMThreadContext *tc, ProfDumpStrs *pds,
-                                    MVMProfileThreadData *ptd) {
+                                    const MVMProfileThreadData *ptd) {
     MVMObject *thread_hash = new_hash(tc);
     MVMObject *thread_gcs  = new_array(tc);
     MVMuint32  i;
@@ -227,6 +261,7 @@ static MVMObject * dump_data(MVMThreadContext *tc) {
     pds.deopt_one       = str(tc, "deopt_one");
     pds.deopt_all       = str(tc, "deopt_all");
     pds.spesh_time      = str(tc, "spesh_time");
+    pds.native_lib      = str(tc, "native library");
 
     /* Build up threads array. */
     /* XXX Only main thread for now. */
@@ -250,6 +285,10 @@ void MVM_profile_start(MVMThreadContext *tc, MVMObject *config) {
 
 /* Ends profiling, builds the result data structure, and returns it. */
 MVMObject * MVM_profile_end(MVMThreadContext *tc) {
+    /* If we have any call frames still on the profile stack, exit them. */
+    while (tc->prof_data->current_call)
+        MVM_profile_log_exit(tc);
+
     /* Disable profiling. */
     /* XXX Needs to account for multiple threads. */
     if (!tc->instance->profiling)
