@@ -171,6 +171,7 @@ typedef struct MVMSpeshInline MVMSpeshInline;
 typedef struct MVMSTable MVMSTable;
 typedef struct MVMStaticFrame MVMStaticFrame;
 typedef struct MVMStaticFrameBody MVMStaticFrameBody;
+typedef struct MVMStaticFrameInstrumentation MVMStaticFrameInstrumentation;
 typedef struct MVMStorageSpec MVMStorageSpec;
 typedef struct MVMString MVMString;
 typedef struct MVMStringBody MVMStringBody;
@@ -192,15 +193,18 @@ typedef struct MVMIOSyncReadable MVMIOSyncReadable;
 typedef struct MVMIOSyncWritable MVMIOSyncWritable;
 typedef struct MVMIOAsyncReadable MVMIOAsyncReadable;
 typedef struct MVMIOAsyncWritable MVMIOAsyncWritable;
+typedef struct MVMIOAsyncWritableTo MVMIOAsyncWritableTo;
 typedef struct MVMIOSeekable MVMIOSeekable;
 typedef struct MVMIOSockety MVMIOSockety;
 typedef struct MVMIOPipeable MVMIOPipeable;
+typedef struct MVMIOIntrospection MVMIOIntrospection;
 typedef struct MVMIOLockable MVMIOLockable;
 typedef struct MVMIOSyncStreamData MVMIOSyncStreamData;
 typedef struct MVMIOSyncPipeData MVMIOSyncPipeData;
 typedef struct MVMDecodeStream MVMDecodeStream;
 typedef struct MVMDecodeStreamBytes MVMDecodeStreamBytes;
 typedef struct MVMDecodeStreamChars MVMDecodeStreamChars;
+typedef struct MVMDecodeStreamSeparators MVMDecodeStreamSeparators;
 typedef struct MVMNativeCallback MVMNativeCallback;
 typedef struct MVMNativeCallbackCacheHead MVMNativeCallbackCacheHead;
 typedef struct MVMJitGraph MVMJitGraph;
